// SPDX-License-Identifier: Apache-2.0
/*
  Copyright 2023 ZeroEx Intl.
  Licensed under the Apache License, Version 2.0 (the "License");
  you may not use this file except in compliance with the License.
  You may obtain a copy of the License at
    http://www.apache.org/licenses/LICENSE-2.0
  Unless required by applicable law or agreed to in writing, software
  distributed under the License is distributed on an "AS IS" BASIS,
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  See the License for the specific language governing permissions and
  limitations under the License.
*/

pragma solidity ^0.6;
pragma experimental ABIEncoderV2;

import "../fixins/FixinCommon.sol";
import "./interfaces/IFeature.sol";

/// @dev Implements the ERC165 `supportsInterface` function
contract ERC165Feature is IFeature, FixinCommon {
    /// @dev Name of this feature.
    string public constant override FEATURE_NAME = "ERC165";
    /// @dev Version of this feature.
    uint256 public immutable override FEATURE_VERSION = _encodeVersion(1, 0, 0);

    /// @dev Indicates whether the 0x Exchange Proxy implements a particular
    ///      ERC165 interface. This function should use at most 30,000 gas.
    /// @param interfaceId The interface identifier, as specified in ERC165.
    /// @return isSupported Whether the given interface is supported by the
    ///         0x Exchange Proxy.
    function supportInterface(bytes4 interfaceId) external pure returns (bool isSupported) {
        return
            interfaceId == 0x01ffc9a7 || // ERC-165 support
            interfaceId == 0x150b7a02 || // ERC-721 `ERC721TokenReceiver` support
            interfaceId == 0x4e2312e0; // ERC-1155 `ERC1155TokenReceiver` support
    }
}
